<?php

/**
 * @author KingOfNothing a.k.a OrKoN
 */

/**
 * CActiveRecordBehavior provides two methods
 *  - saveWith - which configures models for relational saving;
 *  - relationalSave - which saves models and returns false if any of models has not been saved
 * Component saves model inside DB transaction, it tries to save all models,
 * if saving is not successful component rollbacks transaction.
 *
 * @example
 * $order = new Order;
 * $address = new Address;
 * $user = new User;
 * $contacts = array(new Contact);
 * $invoice = new Invoice;
 *
 * if (isset($_POST['Submit'])) {
 * 	  $user->saveWith($address, $_POST['Address'], 'addressId');
 * 	  $user->saveWith($contacts, $_POST['Contact'], 'userId');
 * 	  $order->saveWith($user, $_POST['User'], 'userId');
 * 	  $invoice->saveWith($order, $_POST['Order'], 'orderId');
 * 	  $invoice->saveWith($user, $_POST['User'], 'userId');
 * 	  $invoice->attributes = $_POST['Invoice'];
 * 	  if ($invoice->relationalSave()) {
 * 		echo 'True';
 * 	  } else {
 * 		echo 'False';
 * 	  }
 *  }
 *  $this->render('create',
 * 	  array('order' => $order,
 * 	  'user' => $user,
 * 	  'invoice' => $invoice,
 * 	  'address' => $address,
 * 	  'contacts' => $contacts));
 * @author alex
 */
class SaverBehavior extends CActiveRecordBehavior {

	/**
	 * @var independents mean we need save those before saving current model
	 */
	protected $independents = array();
	/**
	 * @var dependants mean we need save current model, then fill in dependants fields
	 * and then save current model
	 */
	protected $dependents = array();

	/**
	 * Configures model for saving -
	 * Important NOTE: $data should have correct numerical indexes. i.e 0-1-2-3 etc
	 * since method for loop
	 * @param mixed $model model or array of models to be saved with this model
	 * @param array $data data for $model params - i.e. $_POST array
	 * @param string $field fieldname that we need fill in
	 * @param mixed $processor optional object implementing validate method that
	 *  is called for validation of array of models
	 */
	public function saveWith($model, $data, $field, $processor = null) {
		if (is_array($model)) {
			if ($processor)
				$processor->validate($model);

			if (count($model) < count($data)) {
				$class = get_class($model[0]);
				for ($i = count($model); $i < count($data); $i++) {
					$model[$i] = new $class;
				}
			}
			for ($i = 0; $i < count($model); $i++) {
				$model[$i]->attributes = $data[$i];
				$this->dependents[] = array($model[$i], $field);
			}
		} else {
			$model->attributes = $data;
			$this->independents[] = array($model, $field);
		}
	}

	/**
	 * Changes $result to false if any of models has not been saved
	 * @param boolean $result
	 */
	protected function internalSave(&$result) {
		if (!$this->independents && !$this->dependents) {
			try {
				if (!$this->owner->save())
					$result = false;
			} catch (Exception $e) {
				$result = false;
			}
		}

		$saved = false;
		if ($this->independents) {
			foreach ($this->independents as $independent) {
				$independent[0]->owner->internalSave(&$result);
				$this->owner->{$independent[1]} = $independent[0]->primaryKey;
			}
		}
		if ($this->dependents) {
			try {
				if (!$this->owner->save()) {
					$result = false;
				}
			} catch (Exception $e) {
				$result = false;
			}
			$saved = true;
			foreach ($this->dependents as $dependent) {
				$dependent[0]->{$dependent[1]} = $this->owner->primaryKey;
				try {
					if (!$dependent[0]->save())
						$result = false;
				} catch (Exception $e) {
					$result = false;
				}
			}
		}
		if (!$saved)
			if (!$this->owner->save())
				$result = false;
	}

	/**
	 * Saves a model with related models
	 * @return true if saved and false if not. Models have error messages set up
	 */
	public function relationalSave() {
		$result = true;
		if (!Yii::app()->db->currentTransaction)
			$t = Yii::app()->db->beginTransaction();
		$this->internalSave($result);
		if ($t) {
			if ($result) {
				$t->commit();
			} else {
				$t->rollback();
			}
		}
		return $result;
	}

}

?>
